@@ -21,6 +21,7 @@
 
 import apt
 import apt_pkg
+import glob
 import locale
 import os
 import re
@@ -34,8 +35,7 @@ from subprocess import Popen, PIPE
 from .DistUpgradeGettext import gettext as _
 from .DistUpgradeGettext import ngettext
 
-from .utils import inside_chroot, estimate_kernel_size_in_boot
-
+from .utils import inside_chroot
 
 class CacheException(Exception):
     pass
@@ -49,17 +49,40 @@ class CacheExceptionDpkgInterrupted(CacheException):
     pass
 
 
-# the initrd/vmlinuz/abi space required in /boot for each kernel
-# we estimate based on the current kernel size and add a safety marging
-def _set_kernel_initrd_size():
-    size = estimate_kernel_size_in_boot()
-    if size == 0:
-        logging.warn("estimate_kernel_size_in_boot() returned '0'?")
-        size = 28*1024*1024
+def estimate_kernel_initrd_size_in_boot():
+    """estimate the amount of space used by the kernel and initramfs in /boot,
+    including a safety margin
+    """
+    kernel = 0
+    initrd = 0
+    kver = os.uname()[2]
+    for f in glob.glob("/boot/*%s*" % kver):
+        if f == '/boot/initrd.img-%s' % kver:
+            initrd += os.path.getsize(f)
+        # don't include in the estimate any files that are left behind by
+        # an interrupted package manager run
+        elif (f.find('initrd.img') >= 0 or f.find('.bak') >= 0
+              or f.find('.dpkg-') >= 0):
+            continue
+        else:
+            kernel += os.path.getsize(f)
+    if kernel == 0:
+        logging.warning(
+            "estimate_kernel_initrd_size_in_boot() returned '0' for kernel?")
+        kernel = 28*1024*1024
+    if initrd == 0:
+        logging.warning(
+            "estimate_kernel_initrd_size_in_boot() returned '0' for initrd?")
+        initrd = 45*1024*1024
     # add small safety buffer
-    size += 1*1024*1024
-    return size
-KERNEL_INITRD_SIZE = _set_kernel_initrd_size()
+    kernel += 1*1024*1024
+    # safety buffer as a percentage of the existing initrd's size
+    initrd_buffer = 1*1024*1024
+    if initrd * 0.05 > initrd_buffer:
+        initrd_buffer = initrd * 0.05
+    initrd += initrd_buffer
+    return kernel,initrd
+KERNEL_SIZE, INITRD_SIZE = estimate_kernel_initrd_size_in_boot()
 
 
 class FreeSpaceRequired(object):
@@ -126,6 +149,9 @@ class MyCache(apt.Cache):
             not "RELEASE_UPGRADE_NO_RECOMMENDS" in os.environ):
             apt_pkg.config.set("APT::Install-Recommends", "true")
 
+
+        apt_pkg.config.set("APT::AutoRemove::SuggestsImportant", "false")
+
     def _apply_dselect_upgrade(self):
         """ honor the dselect install state """
         for pkg in self:
@@ -230,6 +256,20 @@ class MyCache(apt.Cache):
         """ get the size of the additional required space on the fs """
         return self._depcache.usr_size
     @property
+    def additional_required_space_for_snaps(self):
+        """ get the extra size needed to install the snap replacements """
+        try:
+            # update-manager uses DistUpgradeCache.MyCache as the base class
+            # of its own MyCache version - but without actually calling our
+            # constructor at all. This causes that the MyCache version from
+            # update-manager has no self.quirks attribute while still calling
+            # our default version of checkFreeSpace(). Since extra_snap_space
+            # is only used on dist-upgrades, let's just not care and return 0
+            # in this weird, undocumented case.
+            return self.quirks.extra_snap_space
+        except AttributeError:
+            return 0
+    @property
     def is_broken(self):
         """ is the cache broken """
         return self._depcache.broken_count > 0
@@ -387,6 +427,7 @@ class MyCache(apt.Cache):
                 #raise SystemError("Installing '%s' failed" % pkg)
                 return False
         return True
+
     def mark_upgrade(self, pkg, reason=""):
         logging.debug("Upgrading '%s' (%s)" % (pkg, reason))
         if pkg in self and self[pkg].is_installed:
@@ -395,10 +436,12 @@ class MyCache(apt.Cache):
                 logging.error("Upgrading '%s' failed" % pkg)
                 return False
         return True
+
     def mark_remove(self, pkg, reason=""):
         logging.debug("Removing '%s' (%s)" % (pkg, reason))
         if pkg in self:
             self[pkg].mark_delete()
+
     def mark_purge(self, pkg, reason=""):
         logging.debug("Purging '%s' (%s)" % (pkg, reason))
         if pkg in self:
@@ -501,7 +544,7 @@ class MyCache(apt.Cache):
         # if the free drivers would give us a equally hard time, we would
         # never be able to release
         try:
-            from .NvidiaDetector.nvidiadetector import NvidiaDetection
+            from NvidiaDetector.nvidiadetector import NvidiaDetection
         except (ImportError, SyntaxError) as e:
             # SyntaxError is temporary until the port of NvidiaDetector to
             # Python 3 is in the archive.
@@ -565,7 +608,7 @@ class MyCache(apt.Cache):
         removeEssentialOk = self.config.getlist("Distro", "RemoveEssentialOk")
         # check now
         for pkg in self:
-            # WORKADOUND bug on the CD/python-apt #253255
+            # WORKAROUND bug on the CD/python-apt #253255
             ver = pkg._pcache._depcache.get_candidate_ver(pkg._pkg)
             if ver and ver.priority == 0:
                 logging.error("Package %s has no priority set" % pkg.name)
@@ -613,7 +656,7 @@ class MyCache(apt.Cache):
             # check if we got a new kernel (if we are not inside a
             # chroot)
             if inside_chroot():
-                logging.warn("skipping kernel checks because we run inside a chroot")
+                logging.warning("skipping kernel checks because we run inside a chroot")
             else:
                 self.checkForKernel()
 
@@ -637,14 +680,23 @@ class MyCache(apt.Cache):
             # rationale why it doesn't
             lock.release()
             t.join()
-            # FIXME: change the text to something more useful
+            # the most likely problem is the 3rd party pkgs so don't address
+            # foreignPkgs and devRelease being True
             details =  _("An unresolvable problem occurred while "
-                         "calculating the upgrade.\n\n "
-                         "This can be caused by:\n"
-                         " * Upgrading to a pre-release version of Ubuntu\n"
-                         " * Running the current pre-release version of Ubuntu\n"
-                         " * Unofficial software packages not provided by Ubuntu\n"
-                         "\n")
+                         "calculating the upgrade.\n\n ")
+            if self.config.get("Options", "foreignPkgs") == "True":
+                details += _("This was likely caused by:\n"
+                             " * Unofficial software packages not provided by Ubuntu\n"
+                             "Please use the tool 'ppa-purge' from the ppa-purge \n"
+                             "package to remove software from a Launchpad PPA and \n"
+                             "try the upgrade again.\n"
+                             "\n")
+            elif self.config.get("Options", "foreignPkgs") == "False" and \
+                self.config.get("Options", "devRelease") == "True":
+                details +=  _("This was caused by:\n"
+                              " * Upgrading to a pre-release version of Ubuntu\n"
+                              "This is most likely a transient problem, \n"
+                              "please try again later.\n")
             # we never have partialUpgrades (including removes) on a stable system
             # with only ubuntu sources so we do not recommend reporting a bug
             if partialUpgrade:
@@ -652,7 +704,10 @@ class MyCache(apt.Cache):
                              "please try again later.")
             else:
                 details += _("If none of this applies, then please report this bug using "
-                             "the command 'ubuntu-bug ubuntu-release-upgrader-core' in a terminal.")
+                             "the command 'ubuntu-bug ubuntu-release-upgrader-core' in a terminal. ")
+                details += _("If you want to investigate this yourself the log files in "
+                             "'/var/log/dist-upgrade' will contain details about the upgrade. "
+                             "Specifically, look at 'main.log' and 'apt.log'.")
             # make the error text available again on stdout for the
             # text frontend
             self._stopAptResolverLog()
@@ -673,11 +728,13 @@ class MyCache(apt.Cache):
 
         # check the trust of the packages that are going to change
         untrusted = []
+        downgrade = []
         for pkg in self.get_changes():
             if pkg.marked_delete:
                 continue
             # special case because of a bug in pkg.candidate.origins
             if pkg.marked_downgrade:
+                downgrade.append(pkg.name)
                 for ver in pkg._pkg.version_list:
                     # version is lower than installed one
                     if apt_pkg.version_compare(
@@ -701,8 +758,12 @@ class MyCache(apt.Cache):
             if b:
                 logging.warning("AllowUnauthenticated set!")
                 return True
-        except configparser.NoOptionError as e:
+        except configparser.NoOptionError:
             pass
+        if len(downgrade) > 0:
+            downgrade.sort()
+            logging.error("Packages to downgrade found: '%s'" %
+                          " ".join(downgrade))
         if len(untrusted) > 0:
             untrusted.sort()
             logging.error("Unauthenticated packages found: '%s'" %
@@ -849,7 +910,7 @@ class MyCache(apt.Cache):
             except SystemError as e:
                 # warn here, but don't fail, its possible that meta-packages
                 # conflict (like ubuntu-desktop vs xubuntu-desktop) LP: #775411
-                logging.warn("Can't mark '%s' for upgrade (%s)" % (key, e))
+                logging.warning("Can't mark '%s' for upgrade (%s)" % (key, e))
 
         # check if we have a meta-pkg, if not, try to guess which one to pick
         if not metaPkgInstalled():
@@ -919,7 +980,7 @@ class MyCache(apt.Cache):
         # check if we want to purge
         try:
             purge = self.config.getboolean("Distro", "PurgeObsoletes")
-        except configparser.NoOptionError as e:
+        except configparser.NoOptionError:
             purge = False
 
         # this is a delete candidate, only actually delete,
@@ -938,7 +999,7 @@ class MyCache(apt.Cache):
                 if (pkg.name not in remove_candidates or
                       pkg.name in foreign_pkgs or
                       self._inRemovalBlacklist(pkg.name)):
-                    logging.debug("package '%s' has unwanted removals, skipping" % pkgname)
+                    logging.debug("package '%s' produces an unwanted removal '%s', skipping" % (pkgname, pkg.name))
                     self.restore_snapshot()
                     return False
         except (SystemError, KeyError) as e:
@@ -1082,7 +1143,7 @@ class MyCache(apt.Cache):
                 st = os.statvfs(d)
                 free = st.f_bavail * st.f_frsize
             else:
-                logging.warn("directory '%s' does not exists" % d)
+                logging.warning("directory '%s' does not exists" % d)
                 free = 0
             if fs_id in mnt_map:
                 logging.debug("Dir %s mounted on %s" %
@@ -1099,23 +1160,22 @@ class MyCache(apt.Cache):
         # now calculate the space that is required on /boot
         # we do this by checking how many linux-image-$ver packages
         # are installed or going to be installed
-        space_in_boot = 0
+        kernel_count = 0
         for pkg in self:
             # we match against everything that looks like a kernel
             # and add space check to filter out metapackages
             if re.match("^linux-(image|image-debug)-[0-9.]*-.*", pkg.name):
-                if pkg.marked_install:
-                    logging.debug("%s (new-install) added with %s to boot space" % (pkg.name, KERNEL_INITRD_SIZE))
-                    # multiply the KERNEL_INITRD_SIZE by two as we store a
-                    # copy of the old initrd when creating a new one
-                    space_in_boot += (KERNEL_INITRD_SIZE * 2)
-                # mvo: jaunty does not create .bak files anymore
-                #elif (pkg.marked_upgrade or pkg.is_installed):
-                #    logging.debug("%s (upgrade|installed) added with %s to boot space" % (pkg.name, KERNEL_INITRD_SIZE))
-                #    space_in_boot += KERNEL_INITRD_SIZE # creates .bak
+                # upgrade because early in the release cycle the major version
+                # may be the same or they might be -lts- kernels
+                if pkg.marked_install or pkg.marked_upgrade:
+                    logging.debug("%s (new-install) added with %s to boot space" % (pkg.name, KERNEL_SIZE))
+                    kernel_count += 1
+        # space calculated per LP: #1646222
+        space_in_boot = (kernel_count * KERNEL_SIZE
+                         + (kernel_count + 1) * INITRD_SIZE)
 
         # we check for various sizes:
-        # archivedir is were we download the debs
+        # archivedir is where we download the debs
         # /usr is assumed to get *all* of the install space (incorrect,
         #      but as good as we can do currently + safety buffer
         # /     has a small safety buffer as well
@@ -1141,8 +1201,10 @@ class MyCache(apt.Cache):
 
         # sum up space requirements
         for (dir, size) in [(archivedir, self.required_download),
-                            # plus 50M safety buffer in /usr
                             ("/usr", self.additional_required_space),
+                            # this is only >0 for the deb-to-snap quirks
+                            ("/var", self.additional_required_space_for_snaps),
+                            # plus 50M safety buffer in /usr
                             ("/usr", 50*1024*1024),
                             ("/boot", space_in_boot),
                             ("/tmp", 5*1024*1024),   # /tmp for dkms LP: #427035
@@ -1151,6 +1213,9 @@ class MyCache(apt.Cache):
                             # if snapshots are in use
                             ("/usr", required_for_snapshots),
                            ]:
+            # we are ensuring we have more than enough free space not less
+            if size < 0:
+                continue
             dir = os.path.realpath(dir)
             logging.debug("dir '%s' needs '%s' of '%s' (%f)" % (dir, size, fs_free[dir], fs_free[dir].free))
             fs_free[dir].free -= size
