@@ -8,6 +8,12 @@ export LANG='C.UTF-8'
 dpkg-checkbuilddeps -d 'python3-apt, apt-btrfs-snapshot, parsewiki, python3-feedparser,
 			python3-mock, xvfb, gir1.2-gtk-3.0, python3-gi, python3-nose, pep8, python3-distutils-extra, python3-update-manager'
 
+# make sure the DistUpgrade symlink is good LP: #1824430
+SYMLINK=$(readlink -q DistUpgrade/DistUpgrade)
+if [ $SYMLINK != "." ]; then
+   echo "DistUpgrade/DistUpgrade is not a symlink to itself"
+fi
+
 # run the test-suite
 # echo "Running integrated tests"
 xvfb-run nosetests3
@@ -22,7 +28,7 @@ rm -f ./tests/data-sources-list-test/Ubuntu.mirrors
 (cd utils && ./demotions.py cosmic disco > demoted.cfg)
 # when this gets enabled, make sure to add symlink in DistUpgrade
 # echo "Running lts demotions"
-#(cd utils && ./demotions.py xenial bionic > demoted.cfg.xenial)
+(cd utils && ./demotions.py bionic disco > demoted.cfg.bionic)
 
 # update apt_btrfs_snapshot.py copy, this needs an installed
 # apt-btrfs-snapshot on the build machine
